package bauteile;

import java.awt.Graphics2D;
import java.awt.Color;

import konstanten.Style;

public abstract class Bauteil implements BauteilInterface
{
    protected String beschriftung = "";
    protected int xpos=0,ypos=0,rotation=0;

    //wird erst in den erbenden Klassen implementiert:
    public abstract void zeichnen(Graphics2D g2d); 
    public abstract Bauteil clone(); 

    protected void copyParameter(Bauteil bauteil)
    {
        bauteil.setBeschriftung(this.beschriftung);
        bauteil.setRotation(this.rotation);
        bauteil.setTranslation(this.xpos, this.ypos);        
    }

    public int getX()
    {
        return xpos;
    }
    public int getY()
    {
        return ypos;
    }
    public int getRotation()
    {
        return rotation;
    }
    

    public void setBeschriftung(String beschriftung)
    {
        this.beschriftung = beschriftung;
    }

    public void setRotation(int rotation)
    {
        this.rotation = rotation;
    }

    public void setTranslation(int xpos, int ypos)
    {
        this.xpos = xpos;
        this.ypos = ypos;
    }

    protected void setTransformation(Graphics2D g2d)
    {
         g2d.translate(Style.xoff + (int)Math.round( (double)xpos*Style.rastermass ),
                       Style.yoff + (int)Math.round( (double)ypos*Style.rastermass ));      
         g2d.rotate(0.5*Math.PI*(double)rotation);           
    }

    protected void unsetTransformation(Graphics2D g2d)
    {
         g2d.rotate(-0.5*Math.PI*(double)rotation);           
         g2d.translate(-Style.xoff - (int)Math.round( (double)xpos*Style.rastermass ),
                       -Style.yoff - (int)Math.round( (double)ypos*Style.rastermass ));      
    }

    protected void zeichnePad(Graphics2D g2d, Color farbe, int xraster, int yraster)
    {
        g2d.setColor(farbe);
        g2d.fillOval(
                        (int)Math.round( (double)xraster*Style.rastermass -0.5*Style.bohrung*Style.rastermass ),
                        (int)Math.round( (double)yraster*Style.rastermass -0.5*Style.bohrung*Style.rastermass ),                        
                        (int)Math.round( Style.bohrung*Style.rastermass ),
                        (int)Math.round( Style.bohrung*Style.rastermass )
                    );
    }

    protected void zeichneLinie(Graphics2D g2d, Color farbe, int xraster1, int yraster1,
                                                             int xraster2, int yraster2)
    {
        g2d.setColor(farbe);
        g2d.drawLine(
                        (int)Math.round( (double)xraster1*Style.rastermass ),
                        (int)Math.round( (double)yraster1*Style.rastermass ),                        
                        (int)Math.round( (double)xraster2*Style.rastermass ),
                        (int)Math.round( (double)yraster2*Style.rastermass )                        
                    );
    }

    protected void zeichneBox(Graphics2D g2d, Color farbe, int xraster1, int xraster2,
                                                           int yraster)
    {
        g2d.setColor(farbe);
        g2d.fillRect(
                        (int)Math.round( (double)xraster1*Style.rastermass ),
                        (int)Math.round( (double)yraster*Style.rastermass  -0.6*Style.rastermass),                        
                        (int)Math.round( (double)(xraster2-xraster1)*Style.rastermass ),                        
                        (int)Math.round( 1.2*Style.rastermass )
                    );

        g2d.setColor(Color.BLACK);
        g2d.drawRect(
                        (int)Math.round( (double)xraster1*Style.rastermass ),
                        (int)Math.round( (double)yraster*Style.rastermass  -0.6*Style.rastermass),                        
                        (int)Math.round( (double)(xraster2-xraster1)*Style.rastermass ),                        
                        (int)Math.round( 1.2*Style.rastermass )
                    );
    }


    protected void zeichneBeschriftung(Graphics2D g2d, Color farbe,
                                       int xraster, int yraster)
    {
        g2d.setColor(farbe);
        g2d.setFont(Style.FONT_BESCHRIFTUNG);
        g2d.drawString(
                        beschriftung,
                        Style.XOFF_BESCHRIFTUNG + 
                        (int)Math.round( (double)xraster*Style.rastermass ),
                        Style.YOFF_BESCHRIFTUNG + 
                        (int)Math.round( (double)yraster*Style.rastermass  )                        
                    );
    }

}
